<?php declare(strict_types=1);

namespace LidoNation\Adapayments\App\Services;

use LidoNation\Adapayments\App\Http\Integrations\CoinApi\CoinApiConnector;
use LidoNation\Adapayments\App\Http\Integrations\CoinApi\Requests\GetQuoteRequest as CoinApiGetQuoteRequest;
use LidoNation\Adapayments\App\Http\Integrations\Coinbase\CoinbaseConnector;
use LidoNation\Adapayments\App\Http\Integrations\Coinbase\Requests\GetQuoteRequest as CoinbaseGetQuoteRequest;
use LidoNation\Adapayments\App\Http\Integrations\Coinmarketcap\CoinMarketCapConnector;
use LidoNation\Adapayments\App\Http\Integrations\Coinmarketcap\Requests\GetQuoteRequest as CoinmarketcapGetQuoteRequest;

class AdaUsdQuoteService
{
    public $baseCurrencyName;

    public $baseCurrencySymbol;

    public $quoteCurrencyName;

    public $quoteCurrencySymbol;

    protected $supportedCurrencies = [
        'ADA' => '₳',
        'USD' => '$',
    ];

    public $rate;

    public function __construct($baseName = null, $quoteName = null)
    {
        if (! is_null($baseName) && ! is_null($quoteName)) {
            $this->getExchangeRate($baseName, $quoteName);
        }
    }

    public function getExchangeRate($baseName, $quoteName)
    {
        $isSupported = $this->verifyCurrencySupport([$baseName, $quoteName]);

        if ($isSupported) {
            $this->baseCurrencyName = $baseName;
            $this->quoteCurrencyName = $quoteName;

            $this->setCurrencySymbols();
            $this->getHttpRate($baseName, $quoteName);

            return $this;
        } else {
            echo 'currency pairs in question need to be supported by this module.';
        }
    }

    //verify currency rate exchange support
    public function verifyCurrencySupport($currencies)
    {
        foreach ($currencies as $currency) {
            if (! array_key_exists($currency, $this->supportedCurrencies)) {
                return false;
            }
        }

        return true;
    }

    // sets object's currency symbols based on the query.
    protected function setCurrencySymbols(): self
    {
        $this->baseCurrencySymbol = $this->supportedCurrencies[$this->baseCurrencyName];
        $this->quoteCurrencySymbol = $this->supportedCurrencies[$this->quoteCurrencyName];

        return $this;
    }

    // gets the exchange rate from multiple apis then assigns the public property 'rate' of this class
    protected function getHttpRate(): self
    {
        $ratesArray = $this->fetchHttpRates();
        $validRates = [];
        foreach ($ratesArray as $rate) {
            if ($rate > 0) {
                array_push($validRates, $rate);
            }
        }

        $avgRate = $this->calAvg($validRates);
        $this->rate = $avgRate;

        return $this;
    }

    // call multiple apis and return results in array
    public function fetchHttpRates(): array
    {
        $rateApiProviders = ['coinapi', 'coinbase', 'coinmarketcap'];
        $ratesArray = [];

        foreach ($rateApiProviders as $provider) {
            $rate = $this->{$provider}();
            ($rate !== 0) ? array_push($ratesArray, $rate) : '';
        }

        return $ratesArray;
    }

    //calculates average rate from an array.
    protected function calAvg($rateArray)
    {
        return array_sum($rateArray) / count($rateArray);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // The following are api endpoints functions

    // endpoint
    public function coinapi()
    {
        try {
            $connector = new CoinApiConnector();
            $request = new CoinApiGetQuoteRequest($this->baseCurrencyName, $this->quoteCurrencyName);

            $response = $connector->send($request);

            return $response->json()['rate'];
        } catch (\Throwable $th) {
            return 0;
        }

    }

    // endpoint
    public function coinbase()
    {
        try {
            $connector = new CoinbaseConnector();
            $request = new CoinbaseGetQuoteRequest($this->baseCurrencyName);

            $response = $connector->send($request);

            return $response->json()['data']['rates'][$this->quoteCurrencyName];
        } catch (\Throwable $th) {
            return 0;
        }

    }

    // endpoint
    public function coinmarketcap()
    {
        try {
            $quotes = null;

            // most endpoints need subscription. so we check for
            $connector = new CoinMarketCapConnector();
            $request = new CoinmarketcapGetQuoteRequest(2010);

            $response = $connector->send($request);
            $responseData = $response->json()['data'];
            if (isset($responseData)) {
                $data = (object) ($responseData[2010] ?? null);
                if (isset($data?->quote['USD'])) {
                    $quotes = (object) $data?->quote['USD'];
                }
            }

            // if baseCurrency is USD, then return inverse price
            $inverse = $this->baseCurrencyName === 'ADA' ? false : true;

            return $inverse ? 1 / $quotes->price : $quotes->price;
        } catch (\Throwable $th) {
            return 0;
        }
    }
}
